@@ -46,20 +46,20 @@ SEMANTIC_TAG_FINITE_SET = 258
 
 # Indefinite types begin with their major type ORd with information value 31.
 BEGIN_INDEFINITE_BYTESTRING = struct.pack(
-    r'>B', MAJOR_TYPE_BYTESTRING << 5 | SUBTYPE_INDEFINITE
+    '>B', MAJOR_TYPE_BYTESTRING << 5 | SUBTYPE_INDEFINITE
 )
 BEGIN_INDEFINITE_ARRAY = struct.pack(
-    r'>B', MAJOR_TYPE_ARRAY << 5 | SUBTYPE_INDEFINITE
+    '>B', MAJOR_TYPE_ARRAY << 5 | SUBTYPE_INDEFINITE
 )
 BEGIN_INDEFINITE_MAP = struct.pack(
-    r'>B', MAJOR_TYPE_MAP << 5 | SUBTYPE_INDEFINITE
+    '>B', MAJOR_TYPE_MAP << 5 | SUBTYPE_INDEFINITE
 )
 
-ENCODED_LENGTH_1 = struct.Struct(r'>B')
-ENCODED_LENGTH_2 = struct.Struct(r'>BB')
-ENCODED_LENGTH_3 = struct.Struct(r'>BH')
-ENCODED_LENGTH_4 = struct.Struct(r'>BL')
-ENCODED_LENGTH_5 = struct.Struct(r'>BQ')
+ENCODED_LENGTH_1 = struct.Struct('>B')
+ENCODED_LENGTH_2 = struct.Struct('>BB')
+ENCODED_LENGTH_3 = struct.Struct('>BH')
+ENCODED_LENGTH_4 = struct.Struct('>BL')
+ENCODED_LENGTH_5 = struct.Struct('>BQ')
 
 # The break ends an indefinite length item.
 BREAK = b'\xff'
@@ -262,7 +262,7 @@ else:
         return ord(b[i])
 
 
-STRUCT_BIG_UBYTE = struct.Struct(r'>B')
+STRUCT_BIG_UBYTE = struct.Struct('>B')
 STRUCT_BIG_USHORT = struct.Struct(b'>H')
 STRUCT_BIG_ULONG = struct.Struct(b'>L')
 STRUCT_BIG_ULONGLONG = struct.Struct(b'>Q')
@@ -641,19 +641,28 @@ class sansiodecoder(object):
 
                 elif special == SPECIAL_START_ARRAY:
                     self._collectionstack.append(
-                        {b'remaining': value, b'v': [],}
+                        {
+                            b'remaining': value,
+                            b'v': [],
+                        }
                     )
                     self._state = self._STATE_WANT_ARRAY_VALUE
 
                 elif special == SPECIAL_START_MAP:
                     self._collectionstack.append(
-                        {b'remaining': value, b'v': {},}
+                        {
+                            b'remaining': value,
+                            b'v': {},
+                        }
                     )
                     self._state = self._STATE_WANT_MAP_KEY
 
                 elif special == SPECIAL_START_SET:
                     self._collectionstack.append(
-                        {b'remaining': value, b'v': set(),}
+                        {
+                            b'remaining': value,
+                            b'v': set(),
+                        }
                     )
                     self._state = self._STATE_WANT_SET_VALUE
 
@@ -684,7 +693,10 @@ class sansiodecoder(object):
                     lastc[b'remaining'] -= 1
 
                     self._collectionstack.append(
-                        {b'remaining': value, b'v': newvalue,}
+                        {
+                            b'remaining': value,
+                            b'v': newvalue,
+                        }
                     )
 
                     # self._state doesn't need changed.
@@ -711,7 +723,10 @@ class sansiodecoder(object):
                     lastc[b'remaining'] -= 1
 
                     self._collectionstack.append(
-                        {b'remaining': value, b'v': newvalue,}
+                        {
+                            b'remaining': value,
+                            b'v': newvalue,
+                        }
                     )
 
                     self._state = self._STATE_WANT_SET_VALUE
@@ -775,7 +790,10 @@ class sansiodecoder(object):
                     lastc[b'remaining'] -= 1
 
                     self._collectionstack.append(
-                        {b'remaining': value, b'v': newvalue,}
+                        {
+                            b'remaining': value,
+                            b'v': newvalue,
+                        }
                     )
 
                     self._state = self._STATE_WANT_ARRAY_VALUE
@@ -789,7 +807,10 @@ class sansiodecoder(object):
                     lastc[b'remaining'] -= 1
 
                     self._collectionstack.append(
-                        {b'remaining': value, b'v': newvalue,}
+                        {
+                            b'remaining': value,
+                            b'v': newvalue,
+                        }
                     )
 
                     self._state = self._STATE_WANT_MAP_KEY
@@ -803,7 +824,10 @@ class sansiodecoder(object):
                     lastc[b'remaining'] -= 1
 
                     self._collectionstack.append(
-                        {b'remaining': value, b'v': newvalue,}
+                        {
+                            b'remaining': value,
+                            b'v': newvalue,
+                        }
                     )
 
                     self._state = self._STATE_WANT_SET_VALUE
